//
//  AmityReactionsViewController.swift
//  SampleApp
//
//  Created by Michael Abadi Santoso on 12/24/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import UIKit
import AmitySDK

class AmityReactionsViewController: UIViewController {
    
    private var datasource: IReactionDatasource?
    
    @IBOutlet private weak var tableView: UITableView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        setupTableView()
        datasource?.dataSourceObserver = self
        navigationItem.leftBarButtonItem = UIBarButtonItem(barButtonSystemItem: .cancel, target: self, action: #selector(handleCancelButton))
    }
    
    @objc func handleCancelButton() {
        navigationController?.dismiss(animated: true, completion: nil)
    }
    
    func setDatasource<T>(_ source: T) where T: IReactionDatasource {
        self.datasource = source
    }
    
    private func setupTableView() {
        tableView.delegate = self
        tableView.dataSource = self
        tableView.register(UINib(nibName: "ReactionViewCell", bundle: nil), forCellReuseIdentifier: "ReactionViewCell")
    }
    
    static func makeViewController() -> Self {
        let sb = UIStoryboard(name: "Chats", bundle: nil)
        let vc = sb.instantiateViewController(withIdentifier: "AmityReactionsViewController")
        return vc as! Self
    }
    
}
    
extension AmityReactionsViewController: UITableViewDelegate, UITableViewDataSource {
    
    // MARK: UITableViewDataSource
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        datasource?.numberOfReactions() ?? 0
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        return tableView.dequeueReusableCell(withIdentifier: "ReactionViewCell", for: indexPath)
    }
    
    // MARK: UITableViewDelegate
    func tableView(_ tableView: UITableView, willDisplay cell: UITableViewCell, forRowAt indexPath: IndexPath) {
        guard let datasource = datasource else {
            fatalError("datasource must exist at this point.")
        }
        let reactionViewModel = datasource.reaction(for: indexPath)
        let cell = cell as! ReactionViewCell
        cell.configure(with: reactionViewModel)
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 44
    }
    
    func scrollViewDidScroll(_ scrollView: UIScrollView) {
        // load next page when scrolled to the bottom
        if tableView.contentOffset.y >= (tableView.contentSize.height - tableView.frame.size.height) {
            datasource?.loadMore()
        }
    }
    
}

extension AmityReactionsViewController: DataSourceListener {
    
    func didUpdateDataSource() {
        tableView.reloadData()
    }
    
}
