//
//  ImageManager.swift
//  SampleAppLiveVideo
//
//  Created by Nutchaphon Rewik on 2/12/2563 BE.
//  Copyright © 2563 Amity. All rights reserved.
//

import UIKit
import AmitySDK

class ImageManager {
    
    private var imageCached: NSCache<NSString, UIImage> = NSCache()
    
    static let shared = ImageManager()
    
    private init() {
        // Intentionally left empty
        // This prevents instance iniialization, just use `ImageManager.shared`.
    }
    
    func getThumbnailImage(fromThumbnailFileUrl fileUrl: String, completion: @escaping (UIImage?) -> Void) {
        // Check wether we have the image in the cache.
        if let image = imageCached.object(forKey: fileUrl as NSString) {
            // Image is already in the cached, just grab and show it.
            completion(image)
        } else {
            // We do not have the image yet, try download.
            AppManager.shared.fileRepository?.downloadImageAsData(fromURL: fileUrl,
                                                                  size: .small,
                                                                  completion: { [weak self] image, size, error in
                // Check whether we have got image, if no image we notify nil.
                guard let image = image else {
                    completion(nil)
                    return
                }
                // We've got the image, save it to cache, for later reuse.
                self?.imageCached.setObject(image, forKey: fileUrl as NSString)
                // Notify back the result image.
                completion(image)
            })
        }
    }
    
}
